/*
SCRUTILS.PRG - (C) Copyright 1991 by John H Coleman

SCRUTILS.PRG is a set of utilities that I have put together
recently that I have found to be useful in a number of
programming situations.  These utilities are formatted as User
Defined Functions and can be called from the calling program
using the standard function call.  The routines were written with
Clipper 5.0 in mind but will work with the Summer 87 version as
well with a few modifications.

The functions represented include:

   Visual effects - GROSTR() and POPSTR()
   Screen functions - RSCREEN() and SSCREEN()
   Presentation functions - MENUBOX(), SHADOW(), CENTR(), and COLNUM()
   Menu functions - ASEL1() and ASEL2()

The functions GROSTR() and POPSTR() offer a little variety in the
manner in which strings are written to the screen.

RSCREEN() and SSCREEN() are general replacements for restscreen()
and savescreen() in which string variable for the screen region
carries the region coordinates.  In addition, RSCREEN() returns
the string variable for the region of the screen it is
overwriting.

MENUBOX() automates the process of displaying a three section box
to the screen that contains a region for a header, footer, and the
body of the menu list.

SHADOW() places a shadow at any given location on the screen
using the currently set color.

CENTR() centers a string between any two columns on the specified
line.

COLNUM() returns a three character string that represents the
current color specification - standard, enhansed, and unselected.
The appropriate character can be used directly in the string
variable for a screen region to produce the desired shadow
effect.

ASEL1() and ASEL2() are complementary functions that provides an
achoice() menu that also provides the ability to present a multi-
line message for each item in the achoice() array as it is
highlighted.

Please feel free to try these utilities in your programs.  Any
programs using these routines may be distributed freely with no
royalties.  If you feel that these utilities are of benefit to
you, I ask that you send the paltry sum of $15.00 US to


                       John Coleman
                       38117 S. Julian
                       Mt Clemens, MI 48043

If you have any comments or suggestions regarding these
utilities, you may contact me on Compuserve.  My Compuserve ID
number is  70471,1722.  I don't access the service very often, so
if you do leave a message and I don't get back to you for a few
days, please be patient.
                        
*/




#define StdClr    'N/W,W/R,,,R/W'
#define AltClr    'B+/W'
#define Inverse   'R+/N'
#define Flash     'R+*/N'
#define MenuClr   'B/BG'
#define ShadClr   'N+/GR'
#define True      .t.
#define False     .f.
#define OK        .t.
#define Failed    .f.
#define CrLf      chr(13)+chr(10)
#define Yes       .t.
#define No        .f.
#define Escape    27
#define Enter     13

*-------------------------------------------------------------------
*!*********************************************************************
*!
*!      Procedure: GROSTR(S,R,C)
*!
*!          Calls: POPSTR()       (function  in ?)
*!
*!*********************************************************************
procedure GROSTR(S,R,C)
********************************************************************
* GROSTR presents an animated presentation of a string.  The       *
* string is passed in the parameter S.  The routine opens a box    *
* (one character) in the standard color at the row and column      *
* location passed through R & C.  The string appears to grow from  *
* the box.                                                         *
********************************************************************
local xI, xJ, Dummy
setcolor(StdClr)         // StdClr is a standard color specification
@ R,C say ' '
setcolor(Inverse)        // Inverse is a standard color specification
                         // of any color that will contrast with
                         // the StdClr
for xI:=len(S) to 2 step -1
 @ R,C+1 say substr(S,xI,len(S)-xI+1)
 Dummy:=inkey(.05)
next xI
@ R,C say substr(S,1,1)
return
*--------------------------------------------------------------------
*!*********************************************************************
*!
*!      Procedure: POPSTR(S,R,C)
*!
*!          Calls: WARNING()      (function  in ?)
*!
*!*********************************************************************
procedure POPSTR(S,R,C)
******************************************************************
* POPSTR pops the variable S onto the screen at the location     *
* defined by the passed variables R & C.  The appearance of the  *
* string is accompanied by a sound effect.                       *
******************************************************************
local xI
setcolor(StdClr)       // StdClr is a standard color specification
@ R,C say S
tone(2000,4)
Dummy:=inkey(.4)
setcolor(Inverse)      // Inverse is a standard color specification
                       // of any color that will contrast with
                       // the StdClr
@ R,C say S
return
*--------------------------------------------------------------------
*!*********************************************************************
*!
*!       Function: SSCREEN(R1,C1,R2,C2)()
*!
*!          Calls: RSCREEN()      (function  in ?)
*!
*!*********************************************************************

/*
***************************************************************
* SSCREEN() is intended as a replacement for savescreen().  It
* returns a string variable that represents the specified screen
* region, prefixed with the coordinate variables.  It is intended
* to be used in conjunction with RSCREEN().
*****************************************************************
**/

function SSCREEN(R1,C1,R2,C2)
return chr(R1)+chr(C1)+chr(R2)+chr(C2)+savescreen(R1,C1,R2,C2)
*-------------------------------------------------------------------------
*!*********************************************************************
*!
*!       Function: RSCREEN(SX)()
*!
*!          Calls: SSCREEN()      (function  in ?)
*!
*!*********************************************************************

/*
***************************************************************
* RSCREEN() is intended as a replacement for restscreen() when used
* in conjunction with SSCREEN().  It will extract the window
* coordinates from the variable returned by SSCREEN() and restore
* the identified portion of the screen from the same area is was
* originally located.  RSCREEN() calls SSCREEN() and returns the
* properly formatted string variable for the region of the screen
* where the new image is to be placed.
*
* If Scr1 is a properly formatted string variable af a particular
* screen region, then the following code shows the use of this
* function:
*
* Scr2:=RSCREEN(Scr1)   // Inserts the Scr1 inage into the screen
*        .
*        .
*        .
* RSCREEN(Scr2)        // Restores the original screen
*******************************************************************
**/

function RSCREEN(sX)
local R1,C1,R2,C2,L,sX0
L:=len(sX)
R1:=asc(substr(sX,1,1))
C1:=asc(substr(sX,2,1))
R2:=asc(substr(sX,3,1))
C2:=asc(substr(sX,4,1))
sX:=substr(sX,5,L)
sX0:=SSCREEN(R1,C1,R2,C2)
restscreen(R1,C1,R2,C2,sX)
return sX0
*-------------------------------------------------------------------------
function MENUBOX(R1,C1,R2,C2,T1,L2,T2,L3,T3,S)

********************************************************************
* MENUBOX is a general purpose function that allows a box to be    *
* drawn on the screen at a location specified by the parmeters     *
* list.  Ten arguments are passed - the first five define the box  *
* location, size and line type, the next four define horizontal    *
* dividing lines within the box (if any) and the last parameter,   *
* when true, will draw a shadow around the lower right corner of   *
* the box.                                                         *
********************************************************************
/*
******************************************************************
* R1,C1,R2,C2 are the coordinates of the box to be drawn and do not
* include the shadow portion of the box.
*
* T1 is 0, 1, or 2 and specifies a no line, one line, or two line
* border around the box, respectively.
*
* L2 and L3 specify the number of lines, if any, to be allowed for a
* header or footer, respectively.
*
* T2 and T3 specify the number of lines, if any, that separate the
* header or footer, respectively, from the rest of the box.
*
* S if passed a True value will place a shadow at the lower right
* corner of the box.
*
* MENUBOX() returns a string variable the represents the specified
* region of the screen and includes the shadow if specified.
********************************************************************
**/

local sX0, xClr
if S
 sX0:=SSCREEN(R1,C1,R2+1,C2+2)
else
 sX0:=SSCREEN(R1,C1,R2,C2)
endif
@ R1,C1 clear to R2,C2
do case
 case T1=1
  @ R1,C1 to R2,C2
 case T1=2
  @ R1,C1 to R2,C2 double
endcase

if L2>0   // Draws the title block
 L2++
 do case
  case T2=1
   @ R1+L2,C1 to R1+L2,C2
   do case
    case T1=1
     @ R1+L2,C1 say chr(195)
     @ R1+L2,C2 say chr(180)
    case T1=2
     @ R1+L2,C1 say chr(199)
     @ R1+L2,C2 say chr(182)
   endcase
  case T2=2
   @ R1+L2,C1 to R1+L2,C2 double
   do case
    case T1=1
     @ R1+L2,C1 say chr(198)
     @ R1+L2,C2 say chr(181)
    case T1=2
     @ R1+L2,C1 say chr(204)
     @ R1+L2,C2 say chr(185)
   endcase
 endcase
endif

if L3>0     // Draws the footer block
 L3++
 do case
  case T3=1
   @ R2-L3,C1 to R2-L3,C2
   do case
    case T1=1
     @ R2-L3,C1 say chr(195)
     @ R2-L3,C2 say chr(180)
    case T1=2
     @ R2-L3,C1 say chr(199)
     @ R2-L3,C2 say chr(182)
   endcase
  case T3=2
   @ R2-L3,C1 to R2-L3,C2 double
   do case
    case T1=1
     @ R2-L3,C1 say chr(198)
     @ R2-L3,C2 say chr(181)
    case T1=2
     @ R2-L3,C1 say chr(204)
     @ R2-L3,C2 say chr(185)
   endcase
 endcase
endif

if S         // Places the shadow
* ShadClr:=if(valtype('ShadClr')='U','n+/rg',ShadClr)
 xClr:=setcolor(ShadClr)
 SHADOW(R1+1,C2+1,R2,C2+2)
 SHADOW(R2+1,C1+2,R2+1,C2+2)
 setcolor(xClr)
endif
return sX0
*-------------------------------------------
function SHADOW(R1,C1,R2,C2)

/*
*************************************************************
* SHADOW() places a shadow in the current color scheme at the
* specified coordinate location.
************************************************************
**/

local xClr0, xC, sX
cX:=setcolor()
xC:=substr(COLNUM(cX),1,1)
sX:=savescreen(R1,C1,R2,C2)
sX:=transform(sX,replicate('X'+xC,len(sX)/2))
restscreen(R1,C1,R2,C2,sX)
return NIL
*-------------------------------------------
function CENTR(S,R,C1,C2)

/*
***************************************************************
* CENTR() centers the string, S, on row, R, between colums C1 and
* C2.
***************************************************************
*/

local x1, x2
@ R,C1+((C2-C1)-len(alltrim(S)))/2 say S
return NIL
*-------------------------------------------
function ASEL1

/*
******************************************************************
* ASEL1() when used with ASEL2() will present a menu to the screen
* that allows a multi-line message that corresponds to the
* currently highlighted selection.  The first four parameters
* passed are the screen coordinates of the achoice() list.  
*
* aList is the array of choices.
* aSelItems is the corresponding array of items that are currently
*           selectable within the aList array.
* the next four parameters are the coordinates of the message box.
* aMes is the array corresponding to aList that contains the
*      messages that will be displayed with the menu selection.
*
* The function returns the array element selected as defined by the
* achoice() function.
****************************************************************
**/


local sX1, Rtn
parameters xU,xL,xB,xR,aList,aSelItems,xU2,xL2,xB2,xR2,aMes
sX1:=MENUBOX(xU2,xL2,xB2,xR2,0,0,0,0,0,False)
Rtn:=achoice(xU,xL,xB,xR,aList,aSelItems,'ASEL2')
RSCREEN(sX1)
return Rtn

*---------------------------------------------------
function ASEL2(Mode,Element,RelRow)
/*
************************************************************
* ASEL2() is the UDF() intended to be used with ASEL1().
************************************************************
*/

local Rtn:=2
xKey:=lastkey()
do case
 case Mode=0
  keyboard chr(Escape)
  memoedit(aMes[Element],xU2,xL2,xB2,xR2,False)
  Rtn:=2
 case Mode=1
  tone(100,2)
 case Mode=2
  tone(100,2)
 case Mode=3
  if ((xKey > 47) .and. (xKey < 58)) .or.;
     ((xKey > 64) .and. (xKey < 91)) .or.;
     ((xKey > 96) .and. (xKey < 123))
   Rtn:=3
  elseif xKey=Enter
   Rtn:=1
  endif

 case Mode=4
  Rtn:=0
endcase
return Rtn
*------------------------------------------------------
*!*********************************************************************
*!
*!       Function: COLNUM(C1)()
*!
*!          Calls: SSCREEN()      (function  in ?)
*!               : RSCREEN()      (function  in ?)
*!               : GROSTR()       (function  in ?)
*!
*!*********************************************************************
function COLNUM(C1)

********************************************************************
* Function COLNUM returns a character string that represents the   *
* color setting passed in parameter C1.  The individual characters *
* may be inserted directly into a savescreen() variable which,     *
* when restored, will display the new color setting.  This result  *
* is valuable when it is desired to change the color of the        *
* current screen without redrawing - as in placing a shadow around *
* a box that is popped to the screen.                              *
*                                                                  *
* A three character string is returned  that represent the         *
* standard, enhansed, and unselected colors of the color           *
* specification currently in effect.  The appropriate character    *
* may be inserted directly into the screen variable to get the     *
* desired foreground/background color when restored with           *
* savescren().                                                     *
*                                                                  *
********************************************************************

local n0, n1, n2, n3, c0, c2, c3, s0, s1, p1, l1
n0:=''
s1:=SSCREEN(1,0,1,2)
c0:=setcolor(c1)
c1:=setcolor(c0)
p1:=at(',',c1)
l1:=len(c1)
c2:=substr(c1,p1+1,l1-p1)
p1:=rat(',',c1)
c3:=substr(c1,p1+1,l1-p1)
setcolor(c1)
@ 1,0 say '1'
setcolor(c2)
@ 1,1 say '2'
setcolor(c3)
@ 1,2 say '3'
setcolor(c0)
s0:=savescreen(1,0,1,2)
n1:=substr(s0,2,1)
n2:=substr(s0,4,1)
n3:=substr(s0,6,1)
n0:=n1+n2+n3
RSCREEN(s1)
return n0
*-------------------------------------------------------------------
*===================================================================